#include ".\resource.h"
#include <windows.h>

namespace webdav
{

class ResourceImpl
{
public:
	ResourceImpl();
	virtual ~ResourceImpl(){}

public:
	std::string path_;
	Resource::Type type_;
    __int64 size_;
    time_t modtime_;
    bool is_executable_;
    int is_version_control_;
	std::string error_reason_;
    int error_status_;
    Resource* next_;
};

ResourceImpl::ResourceImpl()
: type_(Resource::TYPE_NORMAL)
  , size_(0)
  , modtime_(0)
  , is_executable_(0)
  , is_version_control_(0)
  , error_status_(0)
  , next_(NULL)
{
}

Resource::Resource()
	: pimpl_(NULL)
{
	pimpl_ = new ResourceImpl;
}

Resource::~Resource()
{
	if(pimpl_ == NULL){
		delete pimpl_;
	}
}

bool Resource::operator < (const Resource& rhs) const
{
	if(GetType() > rhs.GetType()){
		return true;
	}else if(GetType() < rhs.GetType()){
		return false;
	}

	int res = ::CompareString(LOCALE_USER_DEFAULT,
		NORM_IGNORECASE | SORT_STRINGSORT, GetPath(), -1, rhs.GetPath(), -1);

	if(res == CSTR_LESS_THAN){
		return true;
	}

	return false;
}

void Resource::SetPath(const char* path)
{
	pimpl_->path_ = path;
}

const char* Resource::GetPath() const
{
	return pimpl_->path_.c_str();
}

void Resource::SetType(Type type)
{
	pimpl_->type_ = type;
}

Resource::Type Resource::GetType() const
{
	return pimpl_->type_;
}

void Resource::SetSize(__int64 size)
{
	pimpl_->size_ = size;
}

void Resource::SetErrorReason(const char* err)
{
	pimpl_->error_reason_ = err;
}

void Resource::SetErrorStatus(int stt)
{
	pimpl_->error_status_ = stt;
}

void Resource::SetExecutable(bool is_exe)
{
	pimpl_->is_executable_ = is_exe;
}

void Resource::SetVersionControl(int vcr)
{
	pimpl_->is_version_control_ = vcr;
}

void Resource::SetModifiedTime(time_t time)
{
	pimpl_->modtime_ = time;
}

time_t Resource::GetModifiedTime() const
{
	return pimpl_->modtime_;
}

Resource* Resource::Next() const
{
	return pimpl_->next_;
}

void Resource::Insert(Resource* next)
{
	// assert(next != NULL);
	pimpl_->next_ = next;
}

__int64 Resource::GetSize() const
{
	return pimpl_->size_;
}

};
