#ifndef SESSION_H_
#define SESSION_H_

#include <ne_session.h>
#include "uri.h"
#include "webdav.h"
#include <ne_my.h>

namespace webdav
{

class Session
{
public:
	Session();

	virtual ~Session();

	void Connect(const char* scheme,
				 const char* hostname,
				 unsigned int port,
				 ne_progress notify_progress_func = NULL,
				 ne_notify_status notify_status_func = NULL,
				 ne_ssl_verify_fn = NULL,
				 ne_ssl_provide_fn = NULL,
				 WebDAV* webdav = NULL);

	void Connect(const char* scheme,
				 const char* hostname,
				 ne_progress notify_progress_func = NULL,
				 ne_notify_status notify_status_func = NULL,
				 ne_ssl_verify_fn = NULL,
				 ne_ssl_provide_fn = NULL,
				 WebDAV* webdav = NULL);

	bool IsConnected() const;

	void Disconnect();

	void SetUserName(const char* username)
	{
		username_ = username;
	}

	void SetPassword(const char* password)
	{
		password_ = password;
	}

	void SetProxyHost(const char* host)
	{
		proxy_host_ = host;
	}

	void SetProxyPort(UINT port)
	{
		proxy_port_ = port;
	}

	void SetProxyUserName(const std::string& username)
	{
		proxy_username_ = username;
	}

	void SetProxyPassword(const std::string& password)
	{
		proxy_password_ = password;
	}

	const char* GetError() const;

	void SetPersist(bool is_enable = true)
	{
		if(sess_ != NULL){
			ne_set_session_flag(sess_, NE_SESSFLAG_PERSIST, is_enable ? 0 : 1);
		}
	}

	void SetUserAgent(const char* product)
	{
		if(sess_ != NULL){
			ne_set_useragent(sess_, product);
		}
	}

	ne_session* GetSession()
	{
		return sess_;
	}

	ne_ssl_certificate* GetSslServerCert()
	{
		return ne_my_get_server_cert(sess_);
	}
	
	URI GetURI();

	std::string GetScheme() const;

private:
	static int SupplyCredsServer(void *userdata,
								 const char *realm,
								 int attempt,
								 char *username,
								 char *password);

	static int SupplyCredsProxy(void *userdata,
								 const char *realm,
								 int attempt,
								 char *username,
								 char *password);

private:
	ne_session* sess_;
	std::string username_;
	std::string password_;
	std::string proxy_host_;
	UINT proxy_port_;
	std::string proxy_username_;
	std::string proxy_password_;

	bool is_connected_;
};

} // namespace WebDAV

#endif
