#include "webdav.h"
#include "webdavimpl.h"
#include <ne_socket.h>
#include <vector>

namespace webdav
{

WebDAV::WebDAV()
: pimpl_(NULL)
{
	pimpl_ = new WebDAVImpl(this);
}

WebDAV::~WebDAV()
{
	if(pimpl_ != NULL){
		delete pimpl_;
	}
}

int WebDAV::Init()
{
	return ne_sock_init();
}

void WebDAV::Exit()
{
	ne_sock_exit();
}

void WebDAV::ExitThread()
{
	ne_sock_exit_thread();
}

void WebDAV::SetUserName(const char* username)
{
	pimpl_->SetUserName(username);
}

void WebDAV::SetPassword(const char* password)
{
	pimpl_->SetPassword(password);
}

void WebDAV::SetProxy(const char* host, UINT port, const char* username, const char* password)
{
	pimpl_->SetProxy(host, port, username, password);
}

void WebDAV::SetUserAgent(const char* token)
{
	pimpl_->SetUserAgent(token);
}

bool WebDAV::Connect(const char* uri)
{
	return pimpl_->Connect(uri);
}

bool WebDAV::Connect(const char* scheme, const char* host)
{
	return pimpl_->Connect(scheme, host);
}

bool WebDAV::IsConnected() const
{
	return pimpl_->IsConnected();
}

bool WebDAV::GetList(const char* path, Resource** buf, size_t* buf_size)
{
	return pimpl_->GetList(path, buf, buf_size);
}

bool WebDAV::Get(const char* rpath, const char* lpath)
{
	return pimpl_->Get(rpath, lpath);
}

bool WebDAV::Put(const char* rpath, const char* lpath)
{
	return pimpl_->Put(rpath, lpath);
}

bool WebDAV::MakeCollection(const char* path)
{
	return pimpl_->MakeCollection(path);
}

bool WebDAV::Delete(const char* path)
{
	return pimpl_->Delete(path);
}

bool WebDAV::Move(const char* spath, const char* dpath, bool is_overwrite)
{
	return pimpl_->Move(spath, dpath, is_overwrite);
}

bool WebDAV::Copy(const char* spath, const char* dpath, bool is_overwrite)
{
	return pimpl_->Copy(spath, dpath, is_overwrite);
}

void WebDAV::NotifyStatus(webdav::NotifyStatus status, webdav::NotifyInfo* info)
{
}

void WebDAV::NotifyProgress(__int64 progress, __int64 total)
{
}

int WebDAV::NotifyResponse(const char* buffer, size_t buflen)
{
	return 0;
}

size_t WebDAV::NotifyRequest(char* buffer, size_t buflen)
{
	return 0;
}

int WebDAV::VerifyServerCertificate(int failures, SslCertificate* cert)
{
	return 0;
}

void WebDAV::ProvideClientCertificate(char** filename, char** password)
{
}

Status WebDAV::GetLastStatus() const
{
	return pimpl_->GetLastStatus();
}

static std::vector<int> handles;

bool WebDAV::OpenCol(const char* path, int& handle)
{
	handle = handles.size() - 1;

	return true;
}

bool WebDAV::ReadCol(int handle, Resource& res)
{

	return true;
}

void WebDAV::CloseCol(int handle)
{
	return;
}

bool WebDAV::GetFileProperty(const char* path)
{
	return pimpl_->GetFileProperty(path);
}

void WebDAV::DebugInit(FILE *stream, int mask)
{
	FILE* fp = fopen("c:\\ne_debug.log", "ab");

	ne_debug_init(fp, mask);
}

PCCERT_CONTEXT WebDAV::GetSslServerCert()
{
	return pimpl_->GetSslServerCert();
}


} // namespace webdav
