#ifndef WEBDAVIMPL_H_
#define WEBDAVIMPL_H_

#include <windows.h>
#include "session.h"
#include "webdav.h"
#include <ne_request.h>

namespace webdav{

class WebDAVImpl
{
private:
	WebDAV* interface_;
	Session sess_;
	Status last_status_;
	SslCertificate ssl_certificate_;
	PCCERT_CONTEXT m_SslServerCert;

public:
	WebDAVImpl(WebDAV*);
	virtual ~WebDAVImpl(void);

	void SetUserName(const char* username);
	void SetPassword(const char* password);
	void SetProxy(const char* host, UINT port, const char* username = NULL, const char* password = NULL);
	void SetUserAgent(const char* token);

	bool Connect(const char* uri);
	bool Connect(const char* scheme, const char* host, unsigned int port);
	bool Connect(const char* scheme, const char* host);
	bool IsConnected() const;
	bool GetList(const char* path, Resource** buf, size_t* buf_size);
	bool Get(const char* rpath, const char* lpath);
	bool Put(const char* rpath, const char* lpath);
	bool MakeCollection(const char* path);
	bool Delete(const char* path);
	bool Move(const char* spath, const char* dpath, bool is_overwrite);
	bool Copy(const char* spath, const char* dpath, bool is_overwrite);
	bool GetFileProperty(const char* path);

	PCCERT_CONTEXT GetSslServerCert();
	Status GetLastStatus() const {return last_status_;}

private:
	static ssize_t body_fd_send(void* userdata, char* buffer, unsigned int count);

	static void NotifyStatus(void *userdata, ne_session_status status,
                             const ne_session_status_info *info);
	static void NotifyProgress(void* userdata, off64_t progress, off64_t total);
	static int NotifyResponse(void *userdata, const char *buffer, size_t buflen);
	static ssize_t NotifyRequest(void *userdata, char *buffer, size_t buflen);
	static int VerifyServerCertificate(void *userdata,
									   int failures,
									   const ne_ssl_certificate *cert);
	static void ProvideClientCertificate(void *userdata,
										 ne_session *sess,
										 const ne_ssl_dname *const *dname,
										 int dncount);
};

}

#endif
